module DashApi 
  module Auth   
    extend ActiveSupport::Concern

    included do
      
      private      

      def current_user
        unless auth_token.blank?
          jwt_payload = jwt_token 
        else 
          jwt_payload = {role: "guest"}
        end       
        HashWithIndifferentAccess.new(jwt_payload)
      end 
  
      def jwt_token    
        case jwt_token_provider
        when "firebase"            
          DashApi::JsonWebToken.decode_firebase(auth_token) 
        when "jwt"
          DashApi::JsonWebToken.decode(auth_token) 
        else 
          raise "Unknown JWT token provider"
        end 
        rescue JWT::ExpiredSignature
          raise "JWT token has expired"
        rescue JWT::VerificationError, JWT::DecodeError
          raise "Invalid JWT token"     
      end 

      def jwt_token_provider 
        jwt = DashApi::JsonWebToken.decode_unverified(auth_token)
        jwt[:firebase].present? ? "firebase" : "jwt"
      end 
  
      def auth_token
        http_token || params['token']      
      end
    
      def http_token
        if request.headers['Authorization'].present?
          request.headers['Authorization'].split(' ').last
        end
      end  

    end

  end 
end 