require 'net/http'
require 'net/https'

module DashApi 
  module JsonWebToken
    require 'jwt'    
    
    JWT_HASH_ALGORITHM = 'HS256'
    GOOGLE_CERTS_URL = 'https://www.googleapis.com/robot/v1/metadata/x509/securetoken@system.gserviceaccount.com'

    def self.encode(payload:, expiration:)
      payload[:exp] = expiration || 15.minutes.from_now.to_i
      JWT.encode(payload, DashApi.jwt_secret, DashApi.jwt_hash_algorithm || JWT_HASH_ALGORITHM)
    end

    def self.decode(jwt_token)
      jwt = JWT.decode(jwt_token, DashApi.jwt_secret, true, {
        algorithm: DashApi.jwt_algorithm || JWT_HASH_ALGORITHM
      })
      HashWithIndifferentAccess.new(jwt[0])
    end

    def self.decode_firebase(jwt_token)
      firebase_web_key = DashApi.firebase_web_key
      jwt = JWT.decode(jwt_token, firebase_web_key, true, { algorithm: 'RS256' })  do |header|
        url = URI(GOOGLE_CERTS_URL)
        json = JSON.parse(Net::HTTP.get(url))
        public_key = OpenSSL::X509::Certificate.new(json[header['kid']]).public_key
      end
      jwt = jwt[0].merge({ role: "user", provider: "firebase" })
      HashWithIndifferentAccess.new(jwt)
    end 

    def self.decode_unverified(jwt_token)
      HashWithIndifferentAccess.new(JWT.decode(jwt_token, nil, false)[0])
    end
  
  end
end